package com.eni.dvtjeb.metier.interceptors;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ejb.PostActivate;
import javax.ejb.PrePassivate;
import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;

import org.jboss.logging.Logger;

import com.eni.dvtejb.metier.entities.Client;

public class NouvelUtilisateurInterceptor {
  
	private static final Logger log = Logger.getLogger(NouvelUtilisateurInterceptor.class);
	
	@PersistenceContext
	private EntityManager em;
	
	@AroundInvoke  
	public Object controleEmail(InvocationContext ctx) throws Exception  {
		 log.info(" ------------ Debut d'interception  ------------ ");  
		 
		 try {			 
			 /*
			  * 1. Rcuperer le client en cours de cration : c'est le paramtre de la mthode.
			  * 2. Vrifier que son email n'existe pas dj en base
			  * 3. Annuler l'inscription si l'email existe dj, sinon poursuivre l'insertion
			  */
			 
			 Client client = (Client)ctx.getParameters()[0];
			 String email = client.getEmail();
			 log.info("email vaut : " + email);
			 Client doublon = (Client)em.createQuery("select c from Client c where c.email = :email").setParameter("email", email).getSingleResult();
			 
			 if (null != doublon) {
				 log.info(" ----------- Cette adresse email existe dj !");
				 throw new AdresseEmailException("Cette adresse email existe dj !");		
			 }
		 }
		 catch (NoResultException nre){
			 // La mthode getSingleResult() lance une exception de type NoResultException si la requte ne retourne aucun rsultat
			 log.info(" ----------- Exception NoResultException: cet email n'existe pas encore en base ------------- ");
		 }
		 catch(Exception e)  {  
		           throw e;  
		 }  
		 return ctx.proceed(); 
	}
		
	@PostActivate
	  public void postActivate(InvocationContext ic) {
		log.info("Mthode appele : " + ic.getMethod());
	  }

	  @PrePassivate
	  public void prePassivate(InvocationContext ic) {
		log.info("Mthode appele : " + ic.getMethod());
	  }

	  @PreDestroy
	  public void preDestroy(InvocationContext ic) {
		  log.info("Le bean intercept : " + ic.getTarget() + " va tre supprim.");
	  }
	    
	  @PostConstruct
	   public void postConstruct(InvocationContext ic) {
		  log.info("Le bean intercept : " + ic.getTarget() + " a t cre.");
	   }
}
